﻿var http = require('http');
var connect = require('connect');
var bodyParser = require('body-parser'); // parse urlencoded request bodies into req.body
var session = require('express-session');
var ActiveDirectory = require('activedirectory'); // https://www.npmjs.com/package/activedirectory#authenticate
var request = require('request');
var config = require('./config');

function TokenVerify(sToken, res, next) {
    // https://nodejs.org/docs/latest-v5.x/api/http.html#http_class_http_clientrequest
    // https://www.npmjs.com/package/request
    // console.log(sToken);
    request({
        // das HART verdrahten des servers der das Token pruefen kann bzw. die administrative konfiguration ist natuerlich ZWINGEND denn:
        // die vertrauensstellung DIESES 3-Party services zu einem BESTIMMTEN UCServer ist ja die grundlage von allem!
        url: 'http://ws-psi.estos.de:7224/ws/direct/asnTokenVerify', // ucserver.estos.de
        method: 'POST',
        json: { "sToken": sToken }
    }, function (err, result, body) {
        // console.log(body);
        if (err) {
            console.log("TokenVerify failed!");
            // http://stackoverflow.com/questions/20857069/express-redirect-is-unreliable-is-there-caching
            res.writeHead(301, {
                'Cache-Control': 'no-cache', // http://stackoverflow.com/questions/9289058/how-can-i-turn-off-caching-in-http-redirections-in-node-js/9290019
                'Location': 'http://localhost:1337/login'
            });
            return res.end();
        }

        if (result.statusCode == 200) {
            // console.log(body);
            if (null == body.asnUserContact.u8sEntryIDDB) {
                /*
                * wenn das verify OK ist dann die daten fuer den User aus dem decodierten Token liefern
                */
                console.log("(U)ser(P)rincipal(N)ame, AD-Benutzerverwaltung: " + body.u8sUserName);
                return next();
            } else {
                /*
                * wenn das ein user aus dem internen UCServer-Management ist dann user mappen und daten liefern
                * stm ist der ansicht dass ein mapping der user aus verschiedenen Management-Systemen:
                *   - ein sicherheitsrisiko darstellen wuerde
                *   - keiner braucht
                */
                console.log("CtiServerUserName, Integrierte-Benutzerverwaltung: " + body.u8sUserName);
                return next();
            }
        /*
        * sonst fehler (TokenVerify failed)
        */
        } else {
            console.log("TokenVerify failed! present LoginForm");
            // http://stackoverflow.com/questions/20857069/express-redirect-is-unreliable-is-there-caching
            res.writeHead(301, {
                'Cache-Control': 'no-cache', // http://stackoverflow.com/questions/9289058/how-can-i-turn-off-caching-in-http-redirections-in-node-js/9290019
                'Location': 'http://localhost:1337/login'
            });
            return res.end();
        }
    });
}

var app = connect()
/*
* die hier erzeugte session/cookie wird von ProCall (WebBrowserCtrl) behandelt wie von einem echten Browser (IE). d.H.
* - die session haengt am ProCall-Process und NICHT an der instanz eines speziellen WebBrowserCtrl
* - auch wenn der WebTab im ProCall mit UnloadOnHide=true configuriert wurde so wird die session/cookie bei einem TabWechsel NICHT zerstoert.
*/
    .use(session({
        secret: 'wurliwurtz',
        resave: true,
        saveUninitialized: true
    }))
    .use(bodyParser.urlencoded({ extended: false })) // ohne eine explizite configuration u.a. mit dem "extended" attribute gibt es eine deprecated warning

/*
* abholen der credentials ueber eine form, Schritt: 2
* GET, http://ws-psi.estos.de:1337/login
*/
    .use('/login', function (req, res) {
        // https://nodejs.org/api/http.html#http_class_http_serverresponse
        res.writeHead(200, {
            'content-type': 'text/html'
            });
        res.write('<html><head></head><body><h3>Login required</h3><form method="POST" action="/authenticate"><input name="user" type="text"><input name="pass" type="password"><input type="submit" value="submit"></form></body></html>');
        res.end();
    })

/*
* mit der (erfolgreichen) authentication wird die session auf Authenticated gesetzt, Schritt: 3
* GET, http://ws-psi.estos.de:1337/authenticate
*/
    .use('/authenticate', function (req, res) {
        var ad = new ActiveDirectory(config);
        ad.authenticate(req.body.user, req.body.pass, function (err, auth) {
            if (err) {
                // console.log('ERROR: ' + JSON.stringify(err));
                // res.end('ERROR: ' + JSON.stringify(err));
                res.writeHead(200, {
                    'content-type': 'text/html'
                });
                res.end('InvalidCredentials! your session is NOT authenticated!');
                return;
            }

            if (auth) {
            // console.log('Authenticated!');
/*
* an dieser stelle nutzen wir das "express-session" module d.h.:
*   dieses module uebernimmt die aufgabe ein session cookie zu erzeugen.
*   dieses cookie enthaelt nur eine referenz auf unsere in-memory session-data
*/
                req.session.Authenticated = req.body.user;
                res.writeHead(200, {
                    'content-type': 'text/html'
                });
/*
* wenn der WebTab im ProCall mit UnloadOnHide=FALSE configuriert wurde so
* darf man auf dieser seite nicht stehenbleiben denn ein Browser refresh laed nur wieder diese seite.
*/
                res.end('your session is now authenticated! <a href="/">continue</a>');
            } else {
                // console.log('Authentication failed!');
                res.writeHead(200, {
                    'content-type': 'text/html'
                });
                res.end('your session is NOT authenticated!');
            }
        });
    })

/*
* eine statische spa
* demonstriert wie ein script sich ein token abholt um AJAX requests zu authorisieren
* GET, http://ws-psi.estos.de:1337/spa
*/
    .use('/spa', function (req, res) {
        // https://nodejs.org/api/http.html#http_class_http_serverresponse
        res.writeHead(200, {
            'content-type': 'text/html'
            });
        // window.external refer to its CActiveXHost
        res.write('<html><head><script>function window::onload(){ alert(window.external.ContextData.GetUserToken()); }</script></head><body><h3>SPA</h3></body></html>');
        res.end();
    })

/*
* wir stellen die architektur um von Chain auf Switch
* mehrere kollegen halten den ansatz mit EINEM Authentication-Modul das einen Switch enthaelt fuer intuitiver/besser
* Hinweise:
* in der reihenfolge der handler darf dieser Authentication-Handler erst NACH den spezialisierten Handlern fuer /login und /authenticate kommen
*   andernfalls laufen wir ja gleich beim start in eine endlosschleife(redirect)
* dieser handler wird JEDEN UnAuthenticated request sofort nach /Login redirecten, Schritt: 1
*/
    .use(function (req, res, next) {
        // primaer auf session.Authenticated checken
        if (req.session.Authenticated) {
            return next();
        }

        var token = req.headers["x-ucws-token"];
        if (token) {
            TokenVerify(token, res, next);
        } else {
            // no token/credentials
            console.log("no token/credentials! present LoginForm");
            // http://stackoverflow.com/questions/20857069/express-redirect-is-unreliable-is-there-caching
            res.writeHead(301, {
                    'Cache-Control': 'no-cache', // http://stackoverflow.com/questions/9289058/how-can-i-turn-off-caching-in-http-redirections-in-node-js/9290019
                    'Location': 'http://localhost:1337/login'
                });
            return res.end();
        }
    })

    // jetzt ist SICHER Authenticated
    .use(function (req, res) {
        res.writeHead(200, {
            'content-type': 'text/html'
        });
        res.end('SORRY no implicit token for foreigen sites! e.g. <a href="http://www.estos.de/">estos</a>, no API for foreigen sites <a href="http://ws-psi.estos.de/procallsdk/ucweb/connect/sso_ajax.htm">SPA</a><br/>YES there is API for enabled Sites <a href="spa">SPA</a>');
    });
var port = process.env.port || 1337;
http.createServer(app).listen(port);
